<?php
// $Id: flashfield.render.inc,v 1.5 2008/10/25 21:04:41 stuartgreenfield Exp $

/**
 * Generate HTML mark for flash content, using SWF Tools if it is available.
 * If SWF Tools is not present then fall back to direct embedding.
 *
 * This function is called with an array of parameters that define a flashnode and an
 * optional array of parameters for swf tools.
 *
 * @param $flashnode
 *   A structured array that defines an item of Flash content. Must include
 *   keys filepath (path to the file), width, height, substitution (string containing
 *   HTML mark up for the substitution content).
 *   Optional keys flashvars and base are strings defining flashvars and the base
 *   directory respectively.
 *   may be included and they will be passed through to SWFTools.
 *   Key          Comment
 *   filepath     path to an swf file to theme
 *   width        display width of the swf file
 *   height       display height of the movie
 *   substitution substitution content to use if using JavaScript insertion
 *   flashvars    flashvars string to pass to the swf file
 *   base         base parameter to pass to the swf file
 *   embed_params array of parameters for the embedding code
 * @param $options
 *   Optional array of other parameters that will passed through to swf tools theme function
 * @return
 *   An HTML string for rendering the flash content
 */
function theme_flashfield_markup($flashnode, $options = array()) {

  // If using public download then encoding spaces that the upload module may have allowed
  $file = $flashnode['filepath'];
  if (variable_get('file_downloads', FILE_DOWNLOADS_PUBLIC) == FILE_DOWNLOADS_PUBLIC) {
    $file = str_replace(' ', '%20', $file);
  }

  // Generate the url for the file we will be rendering
  $file = file_create_url($file);

  // Get alternate content string - note we do not filter it - we assume this is already done
  $html_alt = t($flashnode['substitution'], array('!default' => FLASHFIELD_DEFAULT_HTML_ALT));

  // Make sure that height and width are integers
  $flashnode['height'] = round($flashnode['height']);
  $flashnode['width'] = round($flashnode['width']);

  // Generate the default parameters array using data that was passed by caller
  $params = $flashnode['embed_params'];

  // Over-ride with user parameters by turning params string in to an array
  $params = array_merge($params, _flashfield_get_params($flashnode['params']));

  // Add base to the parameters array
  $params['base'] = $flashnode['base'];

  // Generate HTML markup, using SWF Tools if available
  if (defined('SWFTOOLS_INSTALLED')) {

    // Put width, height and base in to new $params array
    $params += array(
      'width'  => $flashnode['width'],
      'height' => $flashnode['height'],
    );

    // Create othervars array and merge the alternate HTML string
    $othervars = array_merge($options, array('html_alt' => $html_alt));

    // Create $options array of data to send to SWF Tools
    $options = array(
      'params' => $params,
      'flashvars' => $flashnode['flashvars'],
      'othervars' => $othervars,
    );

    // Return HTML generated by SWF Tools
    return swf($file, $options);
  }

  // Initialise variable to ensure we only show error once if trying to render mp3 or flv with basic embedding
  static $markup_error_shown = false;

  // If file type is flv or mp3 we can't show it without SWF Tools
  if (preg_match('@flv|mp3$@i', $flashnode['filepath'])) {
    if (!$markup_error_shown) {
      drupal_set_message(t('FlashField needs <a href="@swftools">SWF Tools</a> in order to play mp3 or flv files.', array('@swftools' => 'http://drupal.org/project/swftools')), 'warning');
      $markup_error_shown = true;
    }
    return;
  }

  // Prepare height and width strings ready for rendering
  $width_attr = $flashnode['width'] ? ' width="' . $flashnode['width'] . '"' : '';
  $height_attr = $flashnode['height'] ? ' height="' . $flashnode['height'] . '"' : '';

  // Turn parameters array in to a string ready for rendering
  $params_string = '';
  foreach ($params as $name => $value) {
    $params_string .= '<param name="' . $name . '" value="' . $value . '" />' . "\n";
  }

  // Add flashvars to the parameters string
  $params_string .= $flashnode['flashvars'] ? '<param name="flashvars" value="' . $flashnode['flashvars'] . '" />' . "\n" : '' ;

  // Construct W3C compliant output string
  $output  = '<div class="flashnode">' . "\n";
  $output .= '<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"' . $width_attr . $height_attr . '>' . "\n";
  $output .= '<param name="movie" value="' . $file . '" />' . "\n";
  $output .= $params_string;
  $output .= '<!--[if gte IE 7]>-->' . "\n";
  $output .= '<object type="application/x-shockwave-flash" data="' . $file . '"' . $width_attr . $height_attr . '>' . "\n";
  $output .= $params_string;
  $output .= '<!--<![endif]-->' . "\n";
  $output .= $html_alt . "\n";
  $output .= '<!--[if gte IE 7]>-->' . "\n";
  $output .= '</object>' . "\n";
  $output .= '<!--<![endif]-->' . "\n";
  $output .= '</object>' . "\n";
  $output .= '</div>' . "\n";

  return $output;

}


/**
 * Convert a string in the form parameter=value in to an array
 */
function _flashfield_get_params($parameters_string) {
    // parse query string into associative array
    $params = array();
    if ($parameters_string != '') {
      foreach (explode(' ', $parameters_string) as $kvpair) {
        list($key, $value) = explode('=', $kvpair);
        $params[$key] = $value;
      }
    }
  return $params;
}


/**
 * Scale height and width to comply with given maximums
 * Note - $flashnode is passed by reference here so it is manipulated directly
 *
 */
 function _flashfield_scale_to_limits(&$flashnode, $options = array()) {

  // Merge in default options
  $options += array(
    'max_height' => 0,
    'max_width' => 0,
    'scale' => 1,
  );

  // First scale height and width by the scale parameter
  $flashnode['height'] = $flashnode['height'] * $options['scale'];
  $flashnode['width'] = $flashnode['width'] * $options['scale'];

  // Check width does not exceed defined maximum, and scale if required
  if ($options['max_width']) {
    if ($flashnode['width'] > $options['max_width']) {
      $scale = $options['max_width'] / $flashnode['width'];
      $flashnode['width'] = $flashnode['width'] * $scale;
      $flashnode['height'] = $flashnode['height'] * $scale;
    }
  }

  // Now check height does not exceed defined maximum, and scale if required
  if ($options['max_height']) {
    if ($flashnode['height'] > $options['max_height']) {
      $scale = $options['max_height'] / $flashnode['height'];
      $flashnode['width'] = $flashnode['width'] * $scale;
      $flashnode['height'] = $flashnode['height'] * $scale;
    }
  }

  // Round results to integers
  $flashnode['height'] = round($flashnode['height']);
  $flashnode['width'] = round($flashnode['width']);

}


/**
 * Return an array of default parameter settings
 */
function _flashfield_default_params() {

  return array(
    'swliveconnect'     => 'default',
    'play'              => 'true',
    'loop'              => 'true',
    'menu'              => 'false',
    'quality'           => 'autohigh',
    'scale'             => 'showall',
    'align'             => 'l',
    'salign'            => 'tl',
    'wmode'             => 'opaque',
    'bgcolor'           => '#FFFFFF',
    'version'           => '7',
    'allowfullscreen'   => 'true',
    'allowscriptaccess' => 'sameDomain',
  );

}


/**
 * Evalute the flashvars field if it contains PHP code and the input
 * format for the node allows the use of PHP.
 */
function _flashfield_php_flashvars($flashvars, $format = -1) {

  // Get the list of filters for this node
  $filters = filter_list_format($format);

  // Look for module == php to see if we have PHP allowed for this node
  foreach ($filters as $filter) {
    if ($filter->module == 'php') {
      $flashvars = module_invoke($filter->module, 'filter', 'process', $filter->delta, $format, $flashvars);
    }
  }

  // Encode flashvars to make it safe for inclusion on the page
  $flashvars = check_plain($flashvars);

  // But we need to undo conversion of & to &amp;
  $flashvars = str_replace('&amp;', '&', $flashvars);

  return $flashvars;

}
