<?php
// $Id: flashfield.widget.inc,v 1.1 2008/10/25 21:04:41 stuartgreenfield Exp $

function flashfield_widget(&$form, &$form_state, $field, $items, $delta = 0) {

  $element = array(
    '#type' => $field['widget']['type'],
    '#default_value' => isset($items[$delta]) ? $items[$delta] : '',
  );
  return $element;
}


function flashfield_widget_info() {

  return array(
    'flashfield_flashfield' => array(
      'label' => t('Flash content'),
      'field types' => array('flashfield'),
      'multiple values' => CONTENT_HANDLE_CORE,
      'callbacks' => array(
        'default value' => CONTENT_CALLBACK_DEFAULT,
      ),
    ),
  );
}


function flashfield_widget_settings($op, $widget) {

    switch ($op) {
      case 'form':

        // Retrieve the argument that identifies the content type
        $type = arg(3);

        // Retrieve the list of FileFields associated with this content type
        $results = db_query("SELECT field_name FROM {content_node_field_instance} WHERE type_name = '%s' AND widget_type = 'filefield_widget' AND widget_active = 1", $type);

        // Build an array of FileFields ready for the select list
        $filefields = array();
        while ($result = db_fetch_object($results)) {
          $filefields[$result->field_name] = $result->field_name;
        }

        // If no FileFields were found generate a warning message
        if (empty($filefields)) {
          drupal_set_message(t('No FileFields have been found for this content type.'), 'warning');
        }

        $form['upload_field'] = array(
          '#type' => 'select',
          '#title' => t('Name of upload field'),
          '#options' => $filefields,
          '#default_value' => isset($widget['upload_field']) ? $widget['upload_field'] : NULL,
          '#required' => TRUE,
          '#description' => t('FlashField will use the file in this FileField.'),
        );

        $form['node_edit_settings'] = array(
          '#type' => 'fieldset',
          '#title' => t('Node edit form settings'),
          '#collapsible' => true,
          '#collapsed' => true,
          '#weight' => 2,
          '#description' => t('Settings that control the appearance and capabilities of flash content can be shown or hidden on the node edit form. If the fields are hidden then the default values will be used and users will be unable to override them.'),
        );

        $form['node_edit_settings']['show_height_and_width'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show height and width settings'),
          '#default_value' => isset($widget['show_height_and_width']) ? $widget['show_height_and_width'] : TRUE,
          '#description' => t('If this field is hidden then FlashField will attempt to automatically determine the size of content.'),
        );

        $form['node_edit_settings']['show_substitution'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show substitution setting'),
          '#default_value' => isset($widget['show_substitution']) ? $widget['show_substitution'] : FALSE,
          '#description' => t('If this field is shown users can specify custom alternate text.'),
        );

        $form['node_edit_settings']['show_flashvars'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show flashvars setting'),
          '#default_value' => isset($widget['show_flashvars']) ? $widget['show_flashvars'] : FALSE,
          '#description' => t('If this field is shown users can pass a flashvars string to the Flash content.'),
        );

        $form['node_edit_settings']['show_base'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show base setting'),
          '#default_value' => isset($widget['show_base']) ? $widget['show_base'] : FALSE,
          '#description' => t('If this field is shown users can define a custom path for the <em>base</em> parameter. If hidden FlashField will default to the system files directory.'),
        );

        $form['node_edit_settings']['show_params'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show parameters setting'),
          '#default_value' => isset($widget['show_params']) ? $widget['show_params'] : FALSE,
          '#description' => t('If this field is shown users can modify the parameters that are passed to the Flash Player itself.'),
        );

        $form['node_edit_settings']['show_display_mode'] = array(
          '#type' => 'checkbox',
          '#title' => t('Show display mode setting'),
          '#default_value' => isset($widget['show_display_mode']) ? $widget['show_display_mode'] : FALSE,
          '#description' => t('If this field is shown users can choose the display mode. The selection is only respected if the Display fields settings are set to <em>User defined</em>.'),
        );

        $form['size_settings'] = array(
          '#type' => 'fieldset',
          '#title' => t('Sizing and scaling settings'),
          '#collapsible' => true,
          '#collapsed' => true,
          '#weight' => 2,
          '#description' => t('The maximum displayed height and width of a flash movie can be limited by entering non-zero values here. If the movie size is greater than these then the movie will be scaled down when it is displayed. A value of zero means that no scaling will occur. This setting can be useful to ensure that the page layout is not disrupted by a large flash movie.'),
        );

        $form['size_settings']['max_height'] = array(
          '#type' => 'textfield',
          '#title' => t('Maximum displayed height'),
          '#default_value' => isset($widget['max_height']) ? $widget['max_height'] : 0,
          '#element_validate' => array('_flashfield_widget_settings_integer_validate'),
        );

        $form['size_settings']['max_width'] = array(
          '#type' => 'textfield',
          '#title' => t('Maximum displayed width'),
          '#default_value' => isset($widget['max_width']) ? $widget['max_width'] : 0,
          '#element_validate' => array('_flashfield_widget_settings_integer_validate'),
        );

        $form['size_settings']['max_teaser_height'] = array(
          '#type' => 'textfield',
          '#title' => t('Maximum displayed teaser height'),
          '#default_value' => isset($widget['max_teaser_height']) ? $widget['max_teaser_height'] : 0,
          '#element_validate' => array('_flashfield_widget_settings_integer_validate'),
        );

        $form['size_settings']['max_teaser_width'] = array(
          '#type' => 'textfield',
          '#title' => t('Maximum displayed teaser width'),
          '#default_value' => isset($widget['max_teaser_width']) ? $widget['max_teaser_width'] : 0,
          '#element_validate' => array('_flashfield_widget_settings_integer_validate'),
        );

        $form['embed_params'] = array(
          '#type' => 'fieldset',
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
          '#weight' => 3,
          '#title' => t('Embedding settings'),
          '#description' => t('Please refer to <a href="@technote">Adobe technote 12701</a> for more details about each setting.',
                               array('@technote' => 'http://kb.adobe.com/selfservice/viewContent.do?externalId=tn_12701')),
        );

        $form['embed_params']['embed_params_version'] = array(
          '#type' => 'textfield',
          '#size' => '5',
          '#title' => t('Flash version'),
          '#default_value' => isset($widget['embed_params_version']) ? $widget['embed_params_version'] : 7,
        );

        $form['embed_params']['embed_params_play'] = array(
          '#type' => 'checkbox',
          '#title' => t('Play immediately <em>(play)</em>'),
          '#default_value' => isset($widget['embed_params_play']) ? $widget['embed_params_play'] : TRUE,
        );

        $form['embed_params']['embed_params_loop'] = array(
          '#type' => 'checkbox',
          '#title' => t('Loop movie <em>(loop)</em>'),
          '#default_value' => isset($widget['embed_params_loop']) ? $widget['embed_params_loop'] : TRUE,
        );

        $form['embed_params']['embed_params_menu'] = array(
          '#type' => 'checkbox',
          '#title' => t('Full menu <em>(menu)</em>'),
          '#description' => t('Displays the full menu, allowing the user a variety of options to enhance or control playback.'),
          '#default_value' => isset($widget['embed_params_menu']) ? $widget['embed_params_menu'] : FALSE,
        );

        $form['embed_params']['embed_params_allowfullscreen'] = array(
          '#type' => 'checkbox',
          '#title' => t('Allow full screen mode <em>(allowfullscreen)</em>'),
          '#default_value' => isset($widget['embed_params_allowfullscreen']) ? $widget['embed_params_allowfullscreen'] : FALSE,
        );

        $form['embed_params']['embed_params_swliveconnect'] = array(
          '#type' => 'checkbox',
          '#title' => t('Load Java <em>(swliveconnect)</em>'),
          '#default_value' => isset($widget['embed_params_swliveconnect']) ? $widget['embed_params_swliveconnect'] : FALSE,
        );

        $form['embed_params']['embed_params_bgcolor'] = array(
          '#type'  => 'textfield',
          '#title' => t('Background color <em>(bgcolor)</em'),
          '#size'  => 7,
          '#maxlength' => 7,
          '#default_value' => isset($widget['embed_params_bgcolor']) ? $widget['embed_params_bgcolor'] : '#FFFFFF',
          '#description' => t('Hexadecimal background color in the form #RRGGBB.'),
          '#element_validate' => array('_flashfield_widget_settings_hex_color_validate'),
        );

        $form['embed_params']['embed_params_quality'] = array(
          '#type' => 'select',
          '#title' => t('Quality <em>(quality)</em>'),
          '#default_value' => isset($widget['embed_params_quality']) ? $widget['embed_params_quality'] : 'default',
          '#options' => array('default' => 'default', 'low' => 'low', 'autolow' => 'autolow', 'medium' => 'medium', 'high' => 'high', 'autohigh' => 'autohigh', 'best' => 'best'),
        );

        $form['embed_params']['embed_params_scale'] = array(
          '#type' => 'select',
          '#title' => t('Scale <em>(scale)</em>'),
          '#default_value' => isset($widget['embed_params_scale']) ? $widget['embed_params_scale'] : 'default',
          '#options' => array('default' => 'default', 'showall' => 'showall', 'noborder' => 'noborder', 'exactfit' => 'exactfit'),
        );

        $form['embed_params']['embed_params_wmode'] = array(
          '#type' => 'select',
          '#title' => t('Window mode <em>(wmode)</em>'),
          '#default_value' => isset($widget['embed_params_wmode']) ? $widget['embed_params_wmode'] : 'default',
          '#options' => array('default' => 'default', 'window' => 'window',  'opaque' => 'opaque',   'transparent' => 'transparent'),
        );

        $form['embed_params']['embed_params_align'] = array(
          '#type' => 'select',
          '#title' => t('Align <em>(align)</em>'),
          '#default_value' => isset($widget['embed_params_align']) ? $widget['embed_params_align'] : 'default',
          '#options' => array('default' => 'default', 'l' => 'left', 'r' => 'right', 't' => 'top', 'b' => 'bottom'),
        );

        $form['embed_params']['embed_params_salign'] = array(
          '#type' => 'select',
          '#title' => t('salign <em>(salign)</em>'),
          '#default_value' => isset($widget['embed_params_salign']) ? $widget['embed_params_salign'] : 'default',
          '#options' => array('default' => 'default', 'l' => 'left', 'r' => 'right', 't' => 'top', 'b' => 'bottom', 'tl' => 'top left', 'tr' => 'top right', 'bl' => 'bottom left', 'br' => 'bottom right'),
        );

        return $form;

      case 'validate':
        return;

      case 'save':

        // Only run the update if we are not currently installing FlashField
        if (isset($widget['previous_field'])) {
          // We must update the content_type table in case the reference field changed
          $table_name = 'content_type_' . $widget['type_name'];
          $field_name = $widget['field_name'] . '_filefield';
          $new_field_name = $widget['upload_field'];
          db_query("UPDATE {$table_name} SET $field_name = '%s'", $new_field_name);
        }

        return array(
          'upload_field',
          'show_height_and_width',
          'show_substitution',
          'show_flashvars',
          'show_base',
          'show_params',
          'show_display_mode',
          'max_height',
          'max_width',
          'max_teaser_height',
          'max_teaser_width',
          'embed_params_version',
          'embed_params_play',
          'embed_params_loop',
          'embed_params_menu',
          'embed_params_allowfullscreen',
          'embed_params_bgcolor',
          'embed_params_quality',
          'embed_params_scale',
          'embed_params_wmode',
          'embed_params_align',
          'embed_params_salign',
          'embed_params_swliveconnect',
        );
  }
}


/**
 * Validation function for widget settings that must be zero or positive integer values
 * This validation function serves four elements
 */
function _flashfield_widget_settings_integer_validate($element, &$form_state) {

  // Determine which element is being validated
  $parent = $element['#parents'][0];

  // Build an array of strings in case an error message needs to be set
  $description = array(
    'max_height' => t('The maximum height must be zero or a positive integer.'),
    'max_width' => t('The maximum width must be zero or a positive integer.'),
    'max_teaser_height' => t('The maximum teaser height must be zero or a positive integer.'),
    'max_teaser_width' => t('The maximum teaser width must be zero or a positive integer.'),
  );

  // Determine form value for this element
  $value = $form_state['values'][$parent];

  // Validate that the value is either zero or a positive integer
  if (!is_numeric($value) || intval($value) != $value || $value < 0) {
    form_error($element, $description[$parent]);
  }
}


/**
 * Validation function for widget settings that must be a hex color string in form #RRGGBB
 */
function _flashfield_widget_settings_hex_color_validate($element, &$form_state) {

  // Validate that the value matches a hex string definition
  if (!ereg("^#[A-Fa-f0-9]{6}$", $form_state['values']['embed_params_bgcolor'])) {
    form_error($element, t('The background color hex string is not valid.'));
  }
}
